
-- return the window size
function getWindowSize()
	return 616, 330
end

-- return the main font
-- the return is a list of font filename and size options
-- the first one found will be used
function getFont()
	if isMac then
		return { { name="GillSans.ttc", size=14 }, { name="GillSans.dfont", size=14 }, { name="Monaco.dfont", size=12 } }
	else
		return { { name="tahoma.ttf", size=14 }, { name="arial.ttf", size=12 } }
	end
end
-- return the label font
function getLabelFont()
	if isMac then
		return { { name="GillSans.ttc", size=11 }, { name="GillSans.dfont", size=11 }, { name="Monaco.dfont", size=9 } }
	else
		return { { name="tahoma.ttf", size=11 }, { name="arial.ttf", size=9 } }
	end
end
-- return the text field font
function getTextFieldFont()
	if isMac then
		return { { name="GillSans.ttc", size=12 }, { name="GillSans.dfont", size=12 }, { name="Monaco.dfont", size=10 } }
	else
		return { { name="tahoma.ttf", size=12 }, { name="arial.ttf", size=10 } }
	end
end
-- return the delay time display font
function getDelayTimeFont()
	if isMac then
		return { { name="Monaco.dfont", size=30 } }
	else
		return { { name="arial.ttf", size=30 } }
	end
end

-- create and lay out all the controls
function createUI()

	-- load textures
	atlas = texture( "buttons.png" )
	atlas_n = texture( "buttons_n.png" )
	background = texture( "bgtile.png" )
	background_n = texture( "bgtile_n.png" )
	setBackgroundTexture( background, background_n )

	-- constants
	local kKnobSize = 33
	local kBigKnobSize = 91
	local kKnobSpace = 44
	local kTextFieldH = 12
	local kTextFieldHSpace = 20
	local kSliderH = 12
	local kMeterW = 15
	local kMeterH = 77
	local pad = 10.0
	local ww, wh = getWindowSize()

	-- make renderers
	bigKnobRenderer = knobRenderer( atlas, atlas_n, 1, 1, 91 )
	smallKnobRenderer = knobRenderer( atlas, atlas_n, 1, 128-33-1, 33 )
	theSliderRenderer = sliderRenderer( atlas, atlas_n )

	-- set window background colour
	setBackgroundColour( 0.05, 0.05, 0.25 )

	-- set location of the banner
	setBannerPosition( wh - 30 )

	-- set outline colours for groups
	setGroupActiveColour( 0.7, 0.75, 0.8, 0.8 )
	setGroupInactiveColour( 0.4, 0.4, 0.7, 0.5 )
	
	local function knobAndTextField( group, param, label, x, y, size, textH, textW, labelLoc )
		textW = textW or size
		local knobx = x + 0.5 * ( textW - size )
		group:add( knob( param, "", knobx, y+textH+1, size ) )
		group:add( textField( param, label, x, y, textW, textH, labelLoc ) )
	end

	local function bigKnobAndTextField( group, param, label, x, y )
		knobAndTextField( group, param, label, x, y, kBigKnobSize, kTextFieldH, nil, "left" )
	end

	local function smallKnobAndTextField( group, param, label, x, y )
		knobAndTextField( group, param, label, x, y, kKnobSize, kTextFieldH, nil, "below" )
	end
	
	local function sliderAndTextField( group, param, label, x, y, w, h, textH, textW )
		textW = textW or w
		local sliderx = x + 0.5 * ( textW - w )
		group:add( slider( param, sliderx, y+textH+1, w, h, "horizontal", 128-12-1, 128-12-1, 12, 12 ) )
		group:add( textField( param, label, x, y, textW, textH, "left" ) )
	end

	-- make groups and knobs
	--
	
	theKnobRenderer = bigKnobRenderer

	mixGroup = group( "Mix" )
	sliderAndTextField( mixGroup, "Dry Mix", "Dry", 0, kTextFieldHSpace+kSliderH, kBigKnobSize, kSliderH, kTextFieldH )
	bigKnobAndTextField( mixGroup, "Mix", "Mix", 0, kTextFieldHSpace*2+2*kSliderH )

	delayGroup = group( "Delay Time" )
	delayGroup:add( dropDown( "Size", 0, 0, kBigKnobSize, kTextFieldH ) )
	sliderAndTextField( delayGroup, "Spread", "Spread", 0, kTextFieldHSpace+kSliderH, kBigKnobSize, kSliderH, kTextFieldH )
	bigKnobAndTextField( delayGroup, "Delay Time", "Delay", 0, kTextFieldHSpace*2+2*kSliderH )

	feedbackGroup = group( "Regeneration" )
	sliderAndTextField( feedbackGroup, "Colour", "Colour", 0, 0, kBigKnobSize, kSliderH, kTextFieldH )
	sliderAndTextField( feedbackGroup, "Feedback Path", "Path", 0, kTextFieldHSpace+kSliderH, kBigKnobSize, kSliderH, kTextFieldH )
	bigKnobAndTextField( feedbackGroup, "Feedback", "Feedback", 0, kTextFieldHSpace*2+2*kSliderH )

	modDepthGroup = group( "Mod Depth" )
	sliderAndTextField( modDepthGroup, "LFO 1 Spread", "Spread", 0, kTextFieldHSpace+kSliderH, kBigKnobSize, kSliderH, kTextFieldH )
	bigKnobAndTextField( modDepthGroup, "LFO 1 Depth", "Depth", 0, kTextFieldHSpace*2+2*kSliderH )

	modSpeedGroup = group( "Mod Speed" )
	bigKnobAndTextField( modSpeedGroup, "LFO 1 Speed", "Speed", 0, kTextFieldHSpace*2+2*kSliderH )

	theKnobRenderer = smallKnobRenderer

	lfo2Group = group( "LFO 2" )
	smallKnobAndTextField( lfo2Group, "LFO 2 Speed", "Speed", 0*kKnobSpace, 0 )
	smallKnobAndTextField( lfo2Group, "LFO 2 Depth", "Depth", 1*kKnobSpace, 0 )
	smallKnobAndTextField( lfo2Group, "LFO 2 Spread", "Spread", 2*kKnobSpace, 0 )

	envGroup = group( "Envelope Modulation" )
	smallKnobAndTextField( envGroup, "Ducking", "Ducking", 0*kKnobSpace, 0 )
	smallKnobAndTextField( envGroup, "Env->LFO 1 Speed", "Speed", 1*kKnobSpace, 0 )
	smallKnobAndTextField( envGroup, "Env->LFO 1 Depth", "Depth", 2*kKnobSpace, 0 )
	smallKnobAndTextField( envGroup, "Env->LFO 1 Spread", "Spread", 3*kKnobSpace, 0 )

	miscGroup = group( "OSC" )
	smallKnobAndTextField( miscGroup, "OSC Port Offset",	"Port",		0*kKnobSpace, 0 )
	
	-- positions!

	local pad = 10

	local xbase = 20
	local ybase = 30
	local x = xbase
	local y = ybase

	x = x + delayGroup:width() + pad
	envGroup:setPosition( x, y )
	x = x + envGroup:width() + pad

	lfo2Group:setPosition( x, y )
	x = x + lfo2Group:width() + pad
	
	miscGroup:setPosition( x, y )
	x = x + miscGroup:width() + pad
	
	local meterX = x
	local meterY = y - 20

	x = xbase
	y = y + envGroup:height() - 10 + pad

	delayGroup:setPosition( x, y )
	x = x + delayGroup:width() + pad

	feedbackGroup:setPosition( x, y )
	x = x + feedbackGroup:width() + pad

	mixGroup:setPosition( x, y )
	x = x + mixGroup:width() + pad

	modDepthGroup:setPosition( x, y )
	x = x + modDepthGroup:width() + pad

	local prefsX = x
	local prefsY = y

	modSpeedGroup:setPosition( x, y )
	x = x + modSpeedGroup:width() + pad

	-- add directly positioned objects
	--
	
	x = xbase-10
	y = ybase+kTextFieldH

	local displayTimeFont = font( getDelayTimeFont )
	add( delayTimeDisplay( displayTimeFont, x, y, delayGroup:width(), kKnobSize ) )

	-- add the prefs button
	
	x = prefsX
	y = prefsY
	
	add( button( "Prefs", "Prefs", x, y, 100, 30 ) )

	-- meters
	
	x = meterX
	y = meterY
	
	meters = levelMeters( atlas, 128-5, 0, kMeterW, kMeterH )
	meters:add( "envelope", 0, x, y )
	x = x + kMeterW + 5
	meters:add( "envelope", 1, x, y )
	x = x + kMeterW

end		-- of createUI()
